from typing import Any

from sqlalchemy import Text, BigInteger, ForeignKey, Index, func, text
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm import Mapped, mapped_column, relationship

from .base import Base
from .snowflake import Snowflake
from .user import User


class Member(Base):
    __tablename__ = "members"

    id: Mapped[int] = mapped_column(
        BigInteger(), primary_key=True, default=Snowflake.generate_int
    )
    name: Mapped[str] = mapped_column(Text(), nullable=False)

    display_name: Mapped[str | None] = mapped_column(Text(), nullable=True)
    bio: Mapped[str | None] = mapped_column(Text(), nullable=True)

    names: Mapped[list[Any]] = mapped_column(JSONB(), nullable=False, default=[])
    pronouns: Mapped[list[Any]] = mapped_column(JSONB(), nullable=False, default=[])
    fields: Mapped[list[Any]] = mapped_column(JSONB(), nullable=False, default=[])

    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"))
    user: Mapped[User] = relationship(back_populates="members", lazy="immediate")

    __table_args__ = (
        Index(
            "members_user_name_idx", "user_id", func.lower(text("name")), unique=True
        ),
    )
